package legion.muyue.best2048;

import android.util.Log;
import java.util.ArrayList;
import java.util.Random;

public class Game {

    private int[][] gameBoard;
    private Random random;
    private int score = 0;

    public Game() {
        gameBoard = new int[4][4];
        random = new Random();
    }

    public void printArray() {
        for (int[] row : gameBoard) {
            String rowString = String.format("%6d%6d%6d%6d%n", row[0], row[1], row[2], row[3]);
            Log.d("Game", rowString);
        }
        Log.d("Game", "\n");
    }

    public void addNewNumbers() {
        ArrayList<int[]> emptySpaces = new ArrayList<>();

        for (int x = 0; x < 4; x++) {
            for (int y = 0; y < 4; y++) {
                if (gameBoard[x][y] == 0) {
                    emptySpaces.add(new int[]{x, y});
                }
            }
        }

        if (!emptySpaces.isEmpty()) {
            int choice = random.nextInt(emptySpaces.size());
            int[] coordinates = emptySpaces.get(choice);
            int x = coordinates[0];
            int y = coordinates[1];
            int newNumber;
            int randomNumber = random.nextInt(100);
            if (randomNumber < 85) {
                newNumber = 2;
            } else if (randomNumber < 95) {
                newNumber = 4;
            } else {
                newNumber = 8;
            }
            gameBoard[x][y] = newNumber;
        }
    }


    public void pushUp() {
        Log.d("Game", "Pushing Up");
        boolean[] alreadyCombined = new boolean[4];

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4];
            for (int x = 1; x < 4; x++) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    while (currentX > 0 && gameBoard[currentX - 1][y] == 0) {
                        currentX--;
                    }

                    if (currentX == 0) {
                        gameBoard[0][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX - 1][y] != value) {
                        gameBoard[currentX][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX - 1]) {
                        gameBoard[currentX - 1][y] *= 2;
                        score += gameBoard[currentX - 1][y];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentX - 1] = true;
                    } else {
                        gameBoard[currentX][y] = value;
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }

    public void pushDown() {
        Log.d("Game", "Pushing Down");
        boolean[] alreadyCombined = new boolean[4];

        for (int y = 0; y < 4; y++) {
            alreadyCombined = new boolean[4];
            for (int x = 2; x >= 0; x--) {
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentX = x;

                    while (currentX < 3 && gameBoard[currentX + 1][y] == 0) {
                        currentX++;
                    }

                    if (currentX == 3) {
                        gameBoard[3][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[currentX + 1][y] != value) {
                        gameBoard[currentX][y] = value;
                        if (currentX != x)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentX + 1]) {
                        gameBoard[currentX + 1][y] *= 2;
                        score += gameBoard[currentX + 1][y];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentX + 1] = true;
                    } else {
                        gameBoard[currentX][y] = value;
                        if (currentX != x) {
                            gameBoard[x][y] = 0;
                        }
                    }
                }
            }
        }
    }

    public void pushLeft() {
        Log.d("Game", "Pushing Left"); // Pour le débogage
        boolean[] alreadyCombined = new boolean[4]; // Tableau par ligne cette fois

        for (int x = 0; x < 4; x++) { // Itère sur les *lignes* d'abord
            alreadyCombined = new boolean[4]; // Réinitialise pour chaque ligne
            for (int y = 1; y < 4; y++) { // Itère sur les *colonnes*, en commençant par la deuxième
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentY = y; // Suit la colonne

                    // Déplace vers la gauche
                    while (currentY > 0 && gameBoard[x][currentY - 1] == 0) {
                        currentY--;
                    }

                    if (currentY == 0) { // Arrivé tout à gauche
                        gameBoard[x][0] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[x][currentY - 1] != value) { // Case à gauche différente
                        gameBoard[x][currentY] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentY - 1]) { // Case à gauche identique et non fusionnée
                        gameBoard[x][currentY - 1] *= 2; // Fusionne
                        score += gameBoard[x][currentY - 1];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentY - 1] = true; // Marque comme fusionné
                    } else { // Case à gauche identique mais déjà fusionnée
                        gameBoard[x][currentY] = value; // Place à côté
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    }
                }
            }
        }
    }

    public void pushRight() {
        Log.d("Game", "Pushing Right"); // Pour le débogage
        boolean[] alreadyCombined = new boolean[4]; // Tableau par ligne

        for (int x = 0; x < 4; x++) { // Itère sur les *lignes*
            alreadyCombined = new boolean[4];  // Réinitialise pour chaque ligne.
            for (int y = 2; y >= 0; y--) { // Itère sur les *colonnes*, de droite à gauche (avant-dernière vers première)
                if (gameBoard[x][y] != 0) {
                    int value = gameBoard[x][y];
                    int currentY = y; // Suit la colonne

                    // Déplace vers la droite
                    while (currentY < 3 && gameBoard[x][currentY + 1] == 0) {
                        currentY++;
                    }

                    if (currentY == 3) { // Arrivé tout à droite
                        gameBoard[x][3] = value;
                        if (currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (gameBoard[x][currentY + 1] != value) { // Case à droite différente
                        gameBoard[x][currentY] = value;
                        if(currentY != y)
                            gameBoard[x][y] = 0;
                    } else if (!alreadyCombined[currentY + 1]) { // Case à droite identique et non fusionnée
                        gameBoard[x][currentY + 1] *= 2; // Fusionne
                        score += gameBoard[x][currentY + 1];
                        gameBoard[x][y] = 0;
                        alreadyCombined[currentY + 1] = true; // Marque comme fusionné
                    } else { // Case à droite identique mais déjà fusionnée
                        gameBoard[x][currentY] = value; // Place à côté
                        if (currentY != y)
                            gameBoard[x][y] = 0;
                    }
                }
            }
        }
    }
}